@@ -30,30 +30,6 @@ from .base import (
     DesktopNotifierBase,
 )
 
-
-Impl: Type[DesktopNotifierBase]
-
-if os.environ.get("CI", False):
-    # don't attempt to initialise notification center in CIs such as github actions
-    # this may otherwise lead to segfaults on macOS test runners
-    from .dummy import DummyNotificationCenter as Impl
-
-elif platform.system() == "Darwin":
-
-    macos_version, *_ = platform.mac_ver()
-
-    if Version(macos_version) >= Version("10.14.0"):
-        from .macos import CocoaNotificationCenter as Impl
-    else:
-        from .macos_legacy import CocoaNotificationCenterLegacy as Impl
-
-elif platform.system() == "Linux":
-    from .dbus import DBusDesktopNotifier as Impl
-
-else:
-    from .dummy import DummyNotificationCenter as Impl
-
-
 __all__ = [
     "Notification",
     "Button",
@@ -73,27 +49,53 @@ PYTHON_ICON_URI = f"file://{PYTHON_ICON_PATH}"
 default_event_loop_policy = asyncio.DefaultEventLoopPolicy()
 
 
-def run_coro_sync(coro: Coroutine[None, None, T]) -> T:
+def get_implementation(macos_legacy: bool = False) -> Type[DesktopNotifierBase]:
     """
-    Runs the given coroutine and returns the result synchronously. This is used as a
-    wrapper to conveniently convert the async API calls to synchronous ones.
+    Return the backend class depending on the platform and version.
+
+    :param macos_legacy: If ``True``, use the legacy NSUserNotificationCenter on macOS.
+    :returns: A desktop notification backend suitable for the current platform.
+    :raises RuntimeError: when passing ``macos_legacy = True`` on macOS 12.0 and later.
     """
 
-    try:
-        loop = asyncio.get_event_loop()
-    except RuntimeError:
-        # If we are running in a thread, create a new event loop. Use the default
-        # event loop policy so that we don't interfere with anything set in the main
-        # thread.
-        loop = default_event_loop_policy.new_event_loop()
+    if os.environ.get("CI", False):
+        # Don't attempt to initialise notification center in CIs such as github actions
+        # this may otherwise lead to segfaults on macOS test runners.
+        from .dummy import DummyNotificationCenter
+
+        return DummyNotificationCenter
+
+    elif platform.system() == "Darwin":
+
+        macos_version, *_ = platform.mac_ver()
+
+        if Version(macos_version) >= Version("12.0") and macos_legacy:
+            raise RuntimeError(
+                "NSUserNotificationCenter is not available on macOS 12 and later"
+            )
+
+        if Version(macos_version) < Version("10.14.0"):
+            # Always use NSUserNotificationCenter.
+            macos_legacy = True
+
+        if macos_legacy:
+            from .macos_legacy import CocoaNotificationCenterLegacy
+
+            return CocoaNotificationCenterLegacy
+        else:
+            from .macos import CocoaNotificationCenter
+
+            return CocoaNotificationCenter
+
+    elif platform.system() == "Linux":
+        from .dbus import DBusDesktopNotifier
+
+        return DBusDesktopNotifier
 
-    if loop.is_running():
-        future = asyncio.run_coroutine_threadsafe(coro, loop)
-        res = future.result()
     else:
-        res = loop.run_until_complete(coro)
+        from .dummy import DummyNotificationCenter
 
-    return res
+        return DummyNotificationCenter
 
 
 class DesktopNotifier:
@@ -116,6 +118,9 @@ class DesktopNotifier:
         app icon is identified by the bundle ID of the sending program (e.g., Python).
     :param notification_limit: Maximum number of notifications to keep in the system's
         notification center. This may be ignored by some implementations.
+    :param macos_legacy: If True, use the legacy NSUserNotificationCenter backend on
+        macOS 11 and lower. This enables sending notifications from unsigned
+        applications but provides more limited functionality.
     """
 
     def __init__(
@@ -123,11 +128,37 @@ class DesktopNotifier:
         app_name: str = "Python",
         app_icon: Optional[str] = PYTHON_ICON_URI,
         notification_limit: Optional[int] = None,
+        macos_legacy: bool = False,
     ) -> None:
+
+        impl_cls = get_implementation(macos_legacy)
+
         self._lock = RLock()
-        self._impl = Impl(app_name, app_icon, notification_limit)
+        self._impl = impl_cls(app_name, app_icon, notification_limit)
         self._did_request_authorisation = False
 
+        try:
+            self._loop = asyncio.get_event_loop()
+        except RuntimeError:
+            # If we are running in a thread, create a new event loop. Use the default
+            # event loop policy so that we don't interfere with anything set in the main
+            # thread.
+            self._loop = default_event_loop_policy.new_event_loop()
+
+    def _run_coro_sync(self, coro: Coroutine[None, None, T]) -> T:
+        """
+        Runs the given coroutine and returns the result synchronously. This is used as a
+        wrapper to conveniently convert the async API calls to synchronous ones.
+        """
+
+        if self._loop.is_running():
+            future = asyncio.run_coroutine_threadsafe(coro, self._loop)
+            res = future.result()
+        else:
+            res = self._loop.run_until_complete(coro)
+
+        return res
+
     @property
     def app_name(self) -> str:
         """The application name"""
@@ -274,7 +305,7 @@ class DesktopNotifier:
             thread,
         )
 
-        return run_coro_sync(coro)
+        return self._run_coro_sync(coro)
 
     @property
     def current_notifications(self) -> List[Notification]:
